#
# Cesaw-fishing.R,  7 Aug 21

library("circular")
library("colorspace")
library("gnm")
library("plyr")


par(las=1)
par(bty="l")
par(pch="+")


# Basic plot of count of actual times
actual_cnts=function(df)
{
act_cnt=count(df$task_actual_time_minutes)

plot(act_cnt, log="xy",
     	xlim=c(1, 1e3),
	xlab="Actual time")
}

# Plot actuals, try and detect spikes and fit regression models
fit_task_act=function(df, fit_eq=TRUE, work_str="Tasks")
{
   make_fit=function(df_spike, show_components=TRUE)
   {
   df_1000=subset(df_spike, x <= 1000)

   # ta_mod=glm(log(freq) ~ x, data=nospike, subset=1:400)
   # pred=predict(ta_mod, newdata=data.frame(x=1:500))
   # lines(exp(pred))
  
   # This seesm to produce a fit that looks close. 
   ta_mod=gnm(freq ~ instances(Mult(1, Exp(x)), 2)-1,
                   data=df_1000, verbose=FALSE,
                   start=c(2000.0, -0.04, 200.0, -0.001),
                   family=poisson(link="identity"))
   # print(summary(ta_mod))

   if (show_components)
      {
      exp_coef=as.numeric(coef(ta_mod))
      lines(df_1000$x, exp_coef[1]*exp(exp_coef[2]*df_1000$x), col=pal_col[4])
      lines(df_1000$x, exp_coef[3]*exp(exp_coef[4]*df_1000$x), col=pal_col[2])
# c(exp_coef[2]/pred[1], exp_coef[4]/pred[1])
      }
   
   pred=predict(ta_mod)
   lines(df_1000$x, pred, col=pal_col[1])
   return(ta_mod)
   }


# tatm=count(df$task_plan_time_minutes)
tatm=count(df$task_actual_time_minutes)
plot(tatm$x, tatm$freq, log="y", col=pal_col[3],
        xaxs="i",
        xlim=c(1, 400),
        xlab="Actual time (minutes)", ylab=work_str)

# Handle variablity of actuals by having a minimum freq
min_freq=max(3, max(tatm$freq)^0.3)

# Find 'spike' values
# Divide by the value (which will be at least 3) to the left and right
# tatm$spl=c(head(tatm$freq, -1)/pmax(tail(tatm$freq, -1), 5), NA)
# tatm$spr=c(tail(tatm$freq, -1)/pmax(head(tatm$freq, -1), 5), NA)
# Geometric mean of value below and above
tatm$gm=sqrt(pmax(tatm$freq, min_freq)*c(NA, NA, pmax(head(tatm$freq, -2), min_freq)))

# spike_ind=which((tatm$spl > 0.5+0.3*log(tatm$x)) &
#                 (tatm$spr > 0.8+0.3*log(tatm$x)))
spike_ind=which((tatm$gm < 0.8*tatm$freq))
spike=subset(tatm, x %in% tatm$x[spike_ind])
# spike=subset(tatm, x %in% tatm$x[which((tatm$spl > 1.8) &
#                                       (tatm$spr > 2.4))])
points(spike$x, spike$freq, pch="O", col="grey", cex=1.2)
nospike=subset(tatm, !(x %in% tatm$x[spike_ind]))
nospike=subset(nospike, x < 2000)

# plot(nospike$x, nospike$freq, type="b", log="xy",
#         xlim=c(1, 1000))

if (fit_eq)
   {
   nos_mod=make_fit(nospike)
   sp_mod=make_fit(spike, FALSE)
   }
}


person_spike=function(df)
{
if (nrow(df) < 100)
   return(NULL)
fit_task_act(df, FALSE)
}


# How many actuals and unique actuals
dev_unq_act=function(df)
{
tatm=count(df$task_actual_time_minutes)
return(data.frame(num_act=nrow(df), num_unq=nrow(tatm)))
}


cur_dev_unq_act=function(cur_start, df_person)
{
upto_cur=subset(df_person, act_start <= cur_start)
tatm=count(upto_cur$task_actual_time_minutes)
return(data.frame(num_act=nrow(upto_cur), num_unq=nrow(tatm)))
}


# Evolution of one member's actuals/unq
evol_dev_unq_act=function(df)
{
all_starts=df$act_start # not always unique, but ignore this
ev_unq_act=adply(all_starts, 1, cur_dev_unq_act, df)
points(ev_unq_act$num_act, ev_unq_act$num_unq, col=df$col_str[1])
}


# How many actuals and unique actuals for each person in a project
project_unq_act=function(df, col_str)
{
unq_tot=ddply(df, .(person_key), dev_unq_act)

points(unq_tot$num_act, unq_tot$num_unq, col=col_str)

# Quadratic appears to be a more realistic fit, but complicates the numbers
# ut_mod=glm(log(num_unq) ~ log(num_act)+I(log(num_act)^2), data=unq_tot)
ut_mod=glm(log(num_unq) ~ log(num_act), data=unq_tot)
print(summary(ut_mod))
pred=predict(ut_mod, newdata=data.frame(num_act=x_bounds))
lines(x_bounds, exp(pred), col=col_str)

# An exponential model is not remotely close to fitting
# gut_mod=glm(num_unq ~ log(num_act), data=unq_tot)
# print(summary(gut_mod))
# pred=predict(gut_mod, newdata=data.frame(num_act=x_bounds))
# lines(x_bounds, pred, col=col_str)

}


# The evolution of each members actuals over time
evol_proj_unq_act=function(df)
{
# Associate every person with a unique color
u_person=unique(df$person_key)
num_id=length(u_person)
pal_col=rainbow(num_id)
df$num_id=mapvalues(df$person_key, u_person, 1:num_id)
df$col_str=pal_col[df$num_id]

plot(1, type="n", log="xy",
     	xlim=range(x_bounds), ylim=c(1, 500),
        xlab="Actuals", ylab="Unique actuals")

unq_tot=ddply(df, .(person_key), evol_dev_unq_act)

}


act_start_min=function(df)
{
start_min=(df$act_start %% 3600) / 60
plot(start_min, df$task_actual_time_minutes, log="y",
     	xlab="Start minute", ylab="Actual")
}


hour_rose_diag=function(df, name_str)
{
start_cir=circular(df*6, units="degrees", rotation="clock")
rose.diag(start_cir, bins=60, shrink=1.2, prop=7.5, axes=FALSE, col="red")
clk_num=seq(0, 59, by=5)
axis.circular(at=circular(6*clk_num-3.0, units="degrees", rotation="clock"),
		labels=clk_num, cex=0.9)
arrows.circular(mean(start_cir, na.rm=TRUE), y=rho.circular(start_cir, na.rm=TRUE), col=pal_col[2], lwd=2)
text(0.8, 1.05, name_str, cex=1.3)

# sd.circular(start_cir, na.rm=TRUE)
}


# Add the date order in which a task's work sessions occurred
session_order=function(df)
{
sess_ord=order(df$act_start)
df$order=sess_ord
df$num_sess=nrow(df)
return(df)
}


plot_task_sess=function(df, sess_cnt)
{
   sess_points=function(df, col_str)
   {
   act_cnt=count(df$task_actual_time_minutes)
   lines(act_cnt, col=col_str)
   }

pal_col=rainbow(sess_cnt)
# par(mfcol=c(1, sess_cnt))
plot(1, type="n", log="xy",
     	yaxs="i",
     	xlim=c(0.95, 360), ylim=c(1, 150),
	xlab="Actual time", ylab="Sessions")
legend(x="bottomleft", legend=paste0("Session ", 1:sess_cnt), bty="n", fill=pal_col, cex=1.3)

sess=subset(df, num_sess == sess_cnt)
dum=sapply(1:sess_cnt, function(X) sess_points(subset(sess, order == X), pal_col[X]))

}


cnt_sess_pairs=function(df, sess_cnt)
{
   sess_seq=function(df)
   {
   seq=paste(df$task_actual_time_minutes[df$order], collapse=" ")
   }

sess=subset(df, num_sess == sess_cnt)

psp=ddply(sess, .(person_key, plan_item_key), sess_seq)

t=count(psp$V1)
sess_seq=t[order(t$freq, decreasing=TRUE), ]

return(sess_seq)
}


# Return true if number is consider to be 'round'
is_round=function(vec)
{
return((vec %% 5) == 0)
}


dev_uses_rnd=function(df)
{
return(data.frame(num_round=length(which(df$isround)),
		  num_act=nrow(df)))
}


# Standard deviation of a multinomial variable
multinom_sd=function(theta, N)
{
return(sqrt(theta*(1-theta)*N))
}


tf=read.csv("CESAW_task_fact.csv", as.is=TRUE) # Task times
sif=read.csv("CESAW_time_fact.csv", as.is=TRUE) # Session times

tf=sif
tf$task_actual_time_minutes=sif$time_log_delta_minutes
tf$task_actual_time_minutes=sif$time_log_delta_minutes
tf$task_actual_start_date=sif$time_log_start_date
tf$task_actual_complete_date=sif$time_log_end_date

tf=subset(tf, task_actual_time_minutes > 0)

# POSIXct has a weird strcuture that some library functions cannot
# handle, e.g., some plyr functions
tf$act_start_date_time=as.POSIXct(tf$task_actual_start_date, format="%m/%d/%Y %H:%M")
tf$act_start=as.integer(tf$act_start_date_time)
tf$start_min=(tf$act_start %% 3600) / 60
tf$act_end_date_time=as.POSIXct(tf$task_actual_complete_date, format="%m/%d/%Y %H:%M")
tf$act_end=as.integer(tf$act_end_date_time)
tf$end_min=(tf$act_end %% 3600) / 60

tf$isround=is_round(tf$task_actual_time_minutes)

#  table(tf$isround)/nrow(tf)
#    FALSE      TRUE
#0.7045319 0.2954681

p95=subset(tf, project_key == 95)
p614=subset(tf, project_key == 614)
p615=subset(tf, project_key == 615)
p617=subset(tf, project_key == 617)

actual_cnts(tf)

par(mfcol=c(2, 2))

pal_col=rainbow(4)

# actual_cnts(p95)
# actual_cnts(p614)
# actual_cnts(p615)
# actual_cnts(p617)

fit_task_act(p95)
fit_task_act(p614)
fit_task_act(p615, work_str="Sessions")
fit_task_act(p617)


par(mfcol=c(7, 6))
par(mar=c(2, 3, 2, 1))

d_ply(p615, .(person_key), person_spike)

par(mfcol=c(1, 1))
par(mar=c(5, 4, 2, 1))

x_bounds=1:2e3

plot(1, type="n", log="xy",
     	xlim=range(x_bounds), ylim=c(1, 500),
        xlab="Actuals", ylab="Unique actuals")

project_unq_act(p95, pal_col[3])
project_unq_act(p614, pal_col[2])
project_unq_act(p615, pal_col[1])
project_unq_act(p617, pal_col[4])


evol_proj_unq_act(p95)
evol_proj_unq_act(p614)
evol_proj_unq_act(p615)
evol_proj_unq_act(p617)


pal_col=rainbow(2)

start_cnt=count(p615$start_min)
plot(start_cnt, type="l", log="y", col=pal_col[1],
	xlab="Minutes past the hour")
end_cnt=count(p615$end_min)
lines(end_cnt, col=pal_col[2])

legend("topright", c("Start", "End"), bty="n", fill=pal_col, cex=1.4)

par(mfcol=c(1, 2))
par(mar=c(2, 1, 2, 1))

hour_rose_diag(p615$start_min, "Start")
hour_rose_diag(p615$end_min, "End")

smoothScatter(p615$start_min, log(p615$task_actual_time_minutes))
smoothScatter(p615$end_min, log(p615$task_actual_time_minutes))
smoothScatter(p615$start_min, p615$end_min)
lines(c(0, 60), c(0, 60), col="white")


# Do round number patterns appear when tasks are split across multiple sessions?

# (person_key, plan_item_key) provides a unique combination for a task
t_ord=ddply(p615, .(person_key, plan_item_key), session_order)

par(mfcol=c(1, 2))
par(mar=c(4, 4, 2, 1))

plot_task_sess(t_ord, 2)
plot_task_sess(t_ord, 3)


psp=cnt_sess_pairs(t_ord, 2)

rd_cnt=count(subset(p615, isround)$task_actual_time_minutes)
plot(rd_cnt, log="y", col=pal_col[1],
     	xaxs="i",
	xlim=c(0.9, 400),
	xlab="Actual minutes")
nrd_cnt=count(subset(p615, !isround)$task_actual_time_minutes)
points(nrd_cnt, col=pal_col[2])
legend(x="topright", legend=c("Divisible by 5", "Not disvisible by 5"), bty="n", fill=pal_col, cex=1.3)

uses_rnd=ddply(p615, .(person_key), dev_uses_rnd)

par(mfcol=c(1, 1))

pal_col=rainbow(4)

plot(uses_rnd$num_act, uses_rnd$num_round, log="xy", col=pal_col[1],
	xlab="Actuals", ylab="Divisible by five")

ur_mod=glm(log(num_round) ~ log(num_act), data=uses_rnd, subset=(num_round > 5))
summary(ur_mod)
xbounds=1:5e3
pred=predict(ur_mod, newdata=data.frame(num_act=xbounds))
lines(xbounds, exp(pred), col=pal_col[3])

lines(c(2,5000), c(2, 5000), col=pal_col[2])
lines(c(2,5000), 0.2*c(2, 5000), col=pal_col[4])

lines(c(2, 5000), c(0.2*2+multinom_sd(0.2, 2), 0.2*5000+multinom_sd(0.2, 5000)),
     				 lty=2, col=pal_col[4])
lines(c(5, 5000), c(0.2*5-multinom_sd(0.2, 5), 0.2*5000-multinom_sd(0.2, 5000)),
     				 lty=2, col=pal_col[4])


